﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AIOWDMNet;  // the namespace exposes the AIOWDM Class interface 

namespace DIO
{
    public partial class Form1 : Form
    {
        // One and only Device:
        public Int32 CardNum = 0;   // assume the first count connected to AIOWDM is the correct type for this sample
        public UInt32 Channels = 8; 

        bool RunFlag = true;

        public UInt32[] value = new UInt32[] { 0, 0, 0 };  // 3 values used for read write
        public int i = 0; // just for demo purposes its incremented every loop never reset or wraps

        // Interval Timer fou GUI update:
        // (Note: A Windows Forms timer designed for single threaded environments not a System Timer 55 ms min res)
        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();


        public Form1()
        {
            InitializeComponent();

            // Initialize default values:

            // Ping the driver AIOWDM.SYS:
            // This is the one place we actual might use all 16 bits returned for an error code AA55 
            // otherwise we only use an 8 bit byte of data per read and write:
            if (AIOWDM.InPortB(0x61) == 0xAA55) //Driver Call  this is an absolute address no relative address function
            {
                MessageBox.Show("The DLL can't find any attached SYS file (AIOWDM.dll -> AIOWDM.sys)\nEnsure that a board is installed properly: check Device Manager for problems or missing cards.", "Warning");
            }

            textBoxStatus.Text = "This sample outputs a walking bit on Port A of the selected group. It then reads back the value from Port A.  This should be identical to the output value due to on-chip readback. It also displays the Input from Port B.";

            // This will get all the card info:
            // Note; We only need to confirm the index of the card which should be 0 with no address 0 needed
            FindCardsWDM();

            // Add the event and the event handler for the method that will 
            // process the timer event to the timer:
            myTimer.Tick += new EventHandler(TimerEventProcessor);

            // Set the timer interval in miliseconds and start this has a 55ms min resolution:
            myTimer.Interval = 1000;  // sec
        }

        private void Form1_Load(object sender, EventArgs e)
        {
        }

        private void FindCardsWDM()
        {
            RunFlag = true;
            CardNum = 0; // "the first card found"
            if (AIOWDM.GetNumCards() == 0) // no AIOWDM.sys-attached PCI* cards present (includes PCIe, PCIx, cPCI, mPCIe, et al)
            {
                labelCardName.Text = "No Card Found";
                textBoxStatus.Text = "No cards were found!";
                textBoxStatus.Text += "This may mean the card is not installed. ";
                textBoxStatus.Text += "Please check Device Manager for a card and its status";
                RunFlag = false;
            }
        }

        private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
        {
            myTimer.Stop();
            UpdateGUIState();
            myTimer.Enabled = true;
        }

        private void UpdateGUIState()
        {
            // This is the walking bit for the board and display:

            int Status;
            int j;
            char[] msg = new char[] { '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0' };
            StringBuilder s2 = new StringBuilder();
            StringBuilder s3 = new StringBuilder();
            StringBuilder s4 = new StringBuilder();

            value[0] = (UInt32)(1 << i++);

            // Driver Calls:
            Status = AIOWDM.RelOutPortB(CardNum, 0, (Byte)value[0]);  //write a byte
            value[1] = AIOWDM.RelInPortB(CardNum, 0); //read a byte into ushort
            value[2] = AIOWDM.RelInPortB(CardNum, 0 + 1); // read a byte into ushort

            if (Channels > 8)
            {
                Status = AIOWDM.RelOutPortB(CardNum, 0 + 4, (Byte)(value[0] >> 8));
                value[1] = value[1] + (UInt32)(AIOWDM.RelInPortB(CardNum, 0 + 4) << 8);
                value[2] = value[2] + (UInt32)(AIOWDM.RelInPortB(CardNum, 0 + 4 + 1) << 8);
            }


            i %= (int)Channels;

            for (j = 0; j < Channels; j++)
            {
                if (((int)value[0] & (int)(1 << j)) != 0)
                {
                    msg[(Channels - 1) - j] = '1';
                }
                else
                {
                    msg[(Channels - 1) - j] = '0';
                }
            }
            s2.Insert(0, msg);
            textBoxOutputA.Text = s2.ToString();

            for (j = 0; j < Channels; j++)
            {
                if (((int)value[1] & (int)(1 << j)) != 0)
                {
                    msg[(Channels - 1) - j] = '1';
                }
                else
                {
                    msg[(Channels - 1) - j] = '0';
                }
            }
            s3.Insert(0, msg);
            textBoxInputA.Text = s3.ToString();

            for (j = 0; j < Channels; j++)
            {
                if (((int)value[2] & (int)(1 << j)) != 0)
                {
                    msg[(Channels - 1) - j] = '1';
                }
                else
                {
                    msg[(Channels - 1) - j] = '0';
                }
            }
            s4.Insert(0, msg);
            textBoxInputB.Text = s4.ToString();

            Update();

        }

        private void btnPerformIO_Click(object sender, EventArgs e)
        {
            if (btnPerformIO.Text == "Stop I/O")
            {
                myTimer.Stop();
                btnPerformIO.Text = "Perform I/O";
            }
            else
            {

                myTimer.Enabled = true;
                btnPerformIO.Text = "Stop I/O";
            }
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            // Right now this is only called on button click not window close ALtF4 etc:

            // Kill the timer:
            myTimer.Stop();
            myTimer.Enabled = false;
            myTimer.Dispose();

            // Close the form and Application:
            this.Close();
        }


    }
}
